# EXPERIMENTAL
from os import path, getenv
from simple_term_menu import TerminalMenu
import yaml

from .utils import (
    buildMenuOptions,
    createNewSessionId,
    cleanWorkspace,
    prepareWorkspace,
    removeTrailSlash,
)

from .constants import MENU_CURSOR_STYLE, SHOW_SEARCH_HINT, OCPEASY_CONFIG_NAME
from .notify import missingConfigurationFile, ocpeasyConfigFileUpdated


def getModuleTypes(PATH_MODULES: str):
    with open(PATH_MODULES) as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
        sortedModules = list(set(sorted(list(map(lambda x: x["kind"], data)))))
        modulesList = buildMenuOptions(sortedModules)
        terminal_menu = TerminalMenu(
            modulesList,
            title="Select a module type:",
            menu_cursor_style=MENU_CURSOR_STYLE,
            show_search_hint=SHOW_SEARCH_HINT,
        )
        menu_entry_index = terminal_menu.show()
        return sortedModules[menu_entry_index]


def getModulesPerKind(PATH_MODULES: str, moduleKindId: str):
    with open(PATH_MODULES) as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
        filteredRecords = list(
            (filter(lambda x: x["kind"] == moduleKindId, data))
        )  # noqa: E501
        modulesForKind = list(map(lambda x: x["id"], filteredRecords))
        modulesList = buildMenuOptions(modulesForKind)
        terminal_menu = TerminalMenu(
            modulesList,
            title="Select a module type:",
            menu_cursor_style=MENU_CURSOR_STYLE,
            show_search_hint=SHOW_SEARCH_HINT,
        )
        menu_entry_index = terminal_menu.show()
        return filteredRecords[menu_entry_index]


def compose(stageId: str = None, proxy: str = None):
    # TODO: select one or more stage to which module is required

    projectDevPath = getenv("PROJECT_DEV_PATH", None)
    pathProject = "." if not projectDevPath else removeTrailSlash(projectDevPath)

    ocpPeasyConfigFound = False
    ocpPeasyConfigPath = f"{pathProject}/{OCPEASY_CONFIG_NAME}"

    if path.isfile(ocpPeasyConfigPath):
        ocpPeasyConfigFound = True
    else:
        missingConfigurationFile()
        return

    if ocpPeasyConfigFound:
        sessionUuid = createNewSessionId()
        prepareWorkspace(sessionUuid)
        PATH_MODULES = f"/tmp/{sessionUuid}/modules/latest.yml"
        moduleType = getModuleTypes(PATH_MODULES)
        moduleSelected = getModulesPerKind(PATH_MODULES, moduleType)

        excludedKeys = ["module"]
        for excluded in excludedKeys:
            del moduleSelected[excluded]

        globalValues = None
        # get current configuration
        with open(ocpPeasyConfigPath) as ocpPeasyConfigFile:
            deployConfigDict = yaml.load(ocpPeasyConfigFile, Loader=yaml.FullLoader)
            globalValues = dict(deployConfigDict)

            # modulesChanged = False
            for el in globalValues["stages"]:
                modules = el.get("modules", [])
                if stageId and el.get("stageId") == stageId or stageId is None:
                    modules.append(moduleSelected)
                    el["modules"] = modules

        with open(ocpPeasyConfigPath, "w") as ocpPeasyConfigFile:
            ocpPeasyConfigFile.write(yaml.dump(globalValues))

        ocpeasyConfigFileUpdated()
        cleanWorkspace(sessionUuid)

    else:
        missingConfigurationFile()
        return



def missingConfigurationFile():
    print(f"\u274c nocpeasy.yml file does not exist, run `ocpeasy scaffold|init` first")


def stageCreated(stageId: str, pathProject: str):
    print(f"\u2713 new OpenShift stage created ({stageId}) for project [{pathProject}]")


def ocpeasyConfigFileUpdated():
    print(f"\u2713 ocpeasy.yml file refreshed")


def missingStage():
    print(f"\u274c stage doesn't exist")


def ocpeasyStageAssetsGenerated():
    print(f"\u2713 OpenShift assets generated")



# import openshift as oc
from .ocUtils import applyStage
from os import environ
from .utils import buildStageAssets


def deploy(projectId: str, stageId: str):
    # print("OpenShift server version: {}".format(oc.get_server_version()))
    # generateYaml
    buildStageAssets(stageId)
    PREFIX_PROJECT_ROOT = environ.get("PROJECT_DEV_PATH", ".")
    applyStage(projectId, f"{PREFIX_PROJECT_ROOT}/.ocpeasy/{stageId}")

    # Set a project context for all inner `oc` invocations and limit execution to 10 minutes
    # with oc.project(projectId), oc.timeout(10 * 60):
    #     # Print the list of qualified pod names (e.g. ['pod/xyz', 'pod/abc', ...]  in the current project
    #     print(
    #         "Found the following pods in {}: {}".format(
    #             oc.get_project_name(), oc.selector("pods").qnames()
    #         )
    #     )

    # connect to OCP
    # TODO: check if the project has been initialized / ocpeasy.yml exists
    # TODO: check if the selected stage exists in the ocpeasy.yml file
    # (otherwise ask user if it should be created)
    # TODO: get oc path from environment variable, if not simply call `oc`



from os import path, getenv, mkdir
from .utils import (
    removeTrailSlash,
    createNewSessionId,
    cloneStrategyRepository,
    cleanWorkspace,
    getPrompt,
)
from .constants import OCPEASY_CONFIG_NAME, OCPEASY_CONTEXT_PATH
from .notify import missingConfigurationFile, stageCreated
import yaml


def createStage():
    projectDevPath = getenv("PROJECT_DEV_PATH", None)
    pathProject = "." if not projectDevPath else removeTrailSlash(projectDevPath)

    # check if ocpeasy config exists
    ocpPeasyConfigFound = False
    ocpPeasyConfigPath = f"{pathProject}/{OCPEASY_CONFIG_NAME}"

    if path.isfile(ocpPeasyConfigPath):
        ocpPeasyConfigFound = True
    else:
        missingConfigurationFile()
        return

    if ocpPeasyConfigFound:
        sessionId = createNewSessionId()
        deployConfigDict = dict()
        # will contain stage related metadata
        stageConfiguration = dict()
        with open(ocpPeasyConfigPath) as ocpPeasyConfigFile:
            deployConfigDict = yaml.load(ocpPeasyConfigFile, Loader=yaml.FullLoader)
            globalValues = dict(deployConfigDict)
            excludedKeys = ["templateMeta", "stages"]
            for excluded in excludedKeys:
                del globalValues[excluded]

            cloneStrategyRepository(sessionId)
            OCPEASY_DEPLOYMENT_PATH = f"{pathProject}/{OCPEASY_CONTEXT_PATH}"
            try:
                # shutil.rmtree(OCPEASY_DEPLOYMENT_PATH, ignore_errors=True)
                if not path.exists(OCPEASY_DEPLOYMENT_PATH):
                    mkdir(OCPEASY_DEPLOYMENT_PATH)

                # get from CLI (e.g.: --stage=dev) ?
                # TODO: check if stage already exists
                stageId = getPrompt(
                    f"What's the id of your stage (default: development)", "development"
                )

                similarStageId = list(
                    filter(
                        lambda x: (x.get("stageId") == stageId),
                        deployConfigDict["stages"],
                    )
                )
                if len(similarStageId) > 0:
                    print(
                        f"\nOpenShift stage ({stageId}) for project [{pathProject}] exists already \u274c"
                    )
                    return

                ocpProject = getPrompt(f"What's the name of the OpenShift project")
                # TODO: check if the container name already exists for the project
                containerId = getPrompt(
                    f"What's the OpenShift container ID/Name (unique per project)"
                )
                containerRouter = getPrompt(
                    f"What's the route of your application? [{containerId}-{ocpProject}.<hostOcp>]"
                )
                podReplicas = getPrompt(
                    f"What's the number of replicas required for your app?"
                )

                stageConfiguration["stageId"] = stageId
                stageConfiguration["ocpProject"] = ocpProject
                stageConfiguration["containerId"] = containerId
                stageConfiguration["containerRouter"] = containerRouter
                stageConfiguration["podReplicas"] = podReplicas
                stageConfiguration["modules"] = []
                stageConfiguration["dockerfile"] = "./Dockerfile"

            except OSError:
                print("Creation of the directory %s failed" % OCPEASY_CONTEXT_PATH)

            cleanWorkspace(sessionId)
            deployConfigDict = {
                **deployConfigDict,
                "stages": [*deployConfigDict["stages"], stageConfiguration],
            }

        with open(ocpPeasyConfigPath, "w") as ocpPeasyConfigFile:
            ocpPeasyConfigFile.write(yaml.dump(deployConfigDict))

    stageCreated(stageId, pathProject)



from git import Repo
from os import walk
from simple_term_menu import TerminalMenu
import yaml
import shutil

from .utils import (
    buildMenuOptions,
    getPrompt,
    removeTrailSlash,
    createNewSessionId,
    cleanWorkspace,
    prepareWorkspace,
)

from .constants import (
    MENU_CURSOR_STYLE,
    PREFIX_STRATEGY,
    SHOW_SEARCH_HINT,
    OCPEASY_CONFIG_NAME,
    PLATFORM_LABEL,
)


def getStrategyVersions(sessionUuid: str):
    PATH_SESSION = f"/tmp/{sessionUuid}"
    _, folders, _ = next(walk(PATH_SESSION))
    strategies = list(filter(lambda x: x.startswith(PREFIX_STRATEGY), folders))
    strategiesOptions = [
        f'{PLATFORM_LABEL} {el.replace(PREFIX_STRATEGY, "").replace("_", ".")}'
        for el in strategies
    ]
    terminal_menu = TerminalMenu(
        buildMenuOptions(strategiesOptions),
        title="Select an OpenShift strategy:",
        menu_cursor_style=MENU_CURSOR_STYLE,
        show_search_hint=SHOW_SEARCH_HINT,
    )
    menu_entry_index = terminal_menu.show()
    return strategies[menu_entry_index]


def getTechnology(PATH_TEMPLATES: str):
    with open(PATH_TEMPLATES) as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
        sortedTechnologies = list(
            set(sorted(list(map(lambda x: x["technology"], data))))
        )
        technologies = buildMenuOptions(sortedTechnologies)
        terminal_menu = TerminalMenu(
            technologies,
            title="Select a technology:",
            menu_cursor_style=MENU_CURSOR_STYLE,
            show_search_hint=SHOW_SEARCH_HINT,
        )
        menu_entry_index = terminal_menu.show()
        return sortedTechnologies[menu_entry_index]


def getFramework(PATH_TEMPLATES: str, technology: str):
    with open(PATH_TEMPLATES) as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
        filteredRecords = list(
            (filter(lambda x: x["technology"] == technology, data))
        )  # noqa: E501
        frameworks = list(map(lambda x: x["id"], filteredRecords))
        frameworksOptions = buildMenuOptions(frameworks)
        terminal_menu = TerminalMenu(
            frameworksOptions,
            title="Select a framework:",
            menu_cursor_style=MENU_CURSOR_STYLE,
            show_search_hint=SHOW_SEARCH_HINT,
        )
        idx = terminal_menu.show()
        selectedFramework = next(
            filter(lambda x: x["id"] == frameworks[idx], filteredRecords)
        )  # noqa: E501

        return (
            selectedFramework["id"],
            selectedFramework["gitRepository"],
            selectedFramework["profile"],
            selectedFramework["version"],
        )


def confirmSelection():
    options = buildMenuOptions(["Yes", "No"])
    terminal_menu = TerminalMenu(
        options, title="Do you want to confirm project initialization?"
    )  # noqa: E501
    menu_entry_index = terminal_menu.show()
    return scaffold() if menu_entry_index == 1 else True


def getOpenshiftRepositoryMetadata(projectName: str):
    # TODO: add rule to prevent camelcase and special characters
    # TODO: get those parameters in the stage generation
    gitRepository = getPrompt(
        f"Where will reside your code?", "https://www.github.com/user/repo.git"
    )
    gitCredentialsId = getPrompt(
        f"What's the git credential ID?", "gogs-repo-pw"
    )  # noqa: E501
    return (
        gitRepository,
        gitCredentialsId,
    )  # noqa: E501


def scaffold(proxy: str = None):
    scaffoldConfig = {}
    sessionUuid = createNewSessionId()

    prepareWorkspace(sessionUuid, proxy)

    scaffoldConfig["strategy"] = getStrategyVersions(sessionUuid)

    PATH_TEMPLATES = f"/tmp/{sessionUuid}/templates/latest.yml"
    technologySelected = getTechnology(PATH_TEMPLATES)
    scaffoldConfig["technology"] = technologySelected

    frameworkId, templateUri, profile, version = getFramework(
        PATH_TEMPLATES, technologySelected
    )
    scaffoldConfig["frameworkId"] = frameworkId
    scaffoldConfig["templateUri"] = templateUri
    scaffoldConfig["profile"] = profile
    scaffoldConfig["version"] = version

    scaffoldConfig["projectName"] = getPrompt("Select a project name:")

    # configure path project
    relativeProjectPath = getPrompt(
        "Where do you want to generate your project (local)?"
    )

    confirmSelection()

    PATH_PROJECT = (
        f"{removeTrailSlash(relativeProjectPath)}/{scaffoldConfig['projectName']}"
    )

    gitConfig = f"http.proxy={proxy}" if proxy != None else ""
    Repo.clone_from(
        f"{scaffoldConfig['templateUri']}", PATH_PROJECT, config=f"{gitConfig}"
    )
    # TODO: get SHA from head

    shutil.rmtree(f"{PATH_PROJECT}/.git", ignore_errors=True)

    (
        gitRepository,
        gitCredentialsId,
    ) = getOpenshiftRepositoryMetadata(scaffoldConfig["projectName"])

    templateMeta = dict(scaffoldConfig)
    del templateMeta["projectName"]

    ocpeasyConfig = {
        "projectName": scaffoldConfig["projectName"],
        "gitRepository": gitRepository,
        "gitCredentialsId": gitCredentialsId,
        # TODO: add SHA template,
        "templateMeta": {**templateMeta},
        "stages": [],
    }

    with open(f"{PATH_PROJECT}/{OCPEASY_CONFIG_NAME}", "w") as f:
        yaml.dump(ocpeasyConfig, f)

    cleanWorkspace(sessionUuid)



from .constants import (
    ALPHABET_LIST_CHAR,
    EMPTY_STRING,
    BASE_STRATEGIES_REPOSITORY,
    OCPEASY_CONFIG_NAME,
    OCPEASY_CONTEXT_PATH,
    CLI_NAME,
)

from uuid import uuid4
from git import Repo
import shutil
import yaml

from os import getenv, path, walk, mkdir

from .notify import missingConfigurationFile, ocpeasyStageAssetsGenerated

from .__version__ import __version__


def prepareWorkspace(sessionUuid: str, proxy: str = None):
    cloneStrategyRepository(sessionUuid, proxy)


def createNewSessionId():
    return uuid4().hex


def buildMenuOptions(arr):
    counter = 0
    options = []
    canHaveIndex = len(arr) < 35
    for el in arr:
        index = -1
        if canHaveIndex:
            if counter < len(ALPHABET_LIST_CHAR):
                index = ALPHABET_LIST_CHAR[counter]
            else:
                index = f"{counter - len(ALPHABET_LIST_CHAR)}"

        if canHaveIndex:
            options.append(f"[{index}] {el}")
        else:
            options.append(f"{el}")
        counter += 1
    return options


def getPrompt(promptText: str, default=None, rule=None):
    value = ""
    while len(value) == 0:
        value = input(
            f"{promptText} {f'(default: {default})' if default != None else EMPTY_STRING}"  # noqa: E501
        )
        if len(value) == 0 and default:
            value = default
    return value


def removeTrailSlash(uri: str):
    if uri.endswith("/"):
        uri = uri[:-1]
    return uri


def cloneStrategyRepository(sessionId: str, proxy: str = None):
    PATH_SESSION = f"/tmp/{sessionId}"
    kwargs = {"config": f"http.proxy={proxy}"} if proxy is not None else {}
    Repo.clone_from(f"{BASE_STRATEGIES_REPOSITORY}", PATH_SESSION, **kwargs)


def cleanWorkspace(sessionId: str):
    shutil.rmtree(f"/tmp/{sessionId}", ignore_errors=True)


def replaceAll(text: str, dic: dict):
    for i in dic.keys():
        text = text.replace(f"{i}", dic.get(i))
    return text.replace("[", "").replace("]", "")


def buildStageAssets(stageId: str):
    projectDevPath = getenv("PROJECT_DEV_PATH", None)
    pathProject = "." if not projectDevPath else removeTrailSlash(projectDevPath)

    # check if ocpeasy config exists
    ocpPeasyConfigFound = False
    ocpPeasyConfigPath = f"{pathProject}/{OCPEASY_CONFIG_NAME}"

    if path.isfile(ocpPeasyConfigPath):
        ocpPeasyConfigFound = True
    else:
        return missingConfigurationFile()

    if ocpPeasyConfigFound:
        sessionId = createNewSessionId()
        deployConfigDict = dict()
        # will contain stage related metadata
        stageConfiguration = dict()
        # will contain token to be replaced into yml config
        tokenConfiguration = dict()
        # TODO: validate ocpeasy.yml file

        cloneStrategyRepository(sessionId)
        OCPEASY_DEPLOYMENT_PATH = f"{pathProject}/{OCPEASY_CONTEXT_PATH}"

        with open(ocpPeasyConfigPath) as ocpPeasyConfigFile:
            deployConfigDict = dict(
                yaml.load(ocpPeasyConfigFile, Loader=yaml.FullLoader)
            )
            # TODO: get stage corresponding to stageId

            selectedStages = list(
                filter(
                    lambda x: (x.get("stageId") == stageId),
                    deployConfigDict["stages"],
                )
            )

            stageConfiguration = selectedStages[0]

            excludedKeys = ["stageId", "modules"]
            for excluded in excludedKeys:
                del stageConfiguration[excluded]

            tokenConfiguration = {
                **stageConfiguration,
                "generatedBy": f"{CLI_NAME} {__version__}",
            }

            strategyId = deployConfigDict["templateMeta"]["strategy"]
            OCP_PROFILE_PATH = f"/tmp/{sessionId}/{strategyId}/profiles/{deployConfigDict['templateMeta']['profile']}"
            _, _, configFiles = next(walk(OCP_PROFILE_PATH))

            # sample: configFiles = ['bc.yaml', 'svc.yaml', 'dc.yaml', 'route.yaml', 'img.yaml']
            STAGE_CONFIG_ROOT = f"{OCPEASY_DEPLOYMENT_PATH}/{stageId}"

            # will ensure new assets are generated each time
            if path.exists(STAGE_CONFIG_ROOT):
                shutil.rmtree(STAGE_CONFIG_ROOT)

            mkdir(STAGE_CONFIG_ROOT)

            for configFile in configFiles:
                configurationPath = f"{OCP_PROFILE_PATH}/{configFile}"
                with open(configurationPath) as f:
                    configAsDict = yaml.load(f, Loader=yaml.FullLoader)
                    ocpContextYaml = yaml.dump(configAsDict)
                    ocpContextBuild = replaceAll(ocpContextYaml, tokenConfiguration)
                    # generate yaml files in .ocpeasy/{stage}/{configFile}
                    print(ocpContextBuild)
                    stageConfigFile = f"{STAGE_CONFIG_ROOT}/{configFile}"
                    with open(stageConfigFile, "w") as configTarget:
                        configTarget.write(ocpContextBuild)

            cleanWorkspace(sessionId)

        return ocpeasyStageAssetsGenerated()



import fire
import sys
import os

PARENT_FOLDER = ".."
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__)))
)
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PARENT_FOLDER)))

from ocpeasy.scaffold import scaffold  # noqa E402, F401
from ocpeasy.createStage import createStage  # noqa E402, F401
from ocpeasy.deploy import deploy  # noqa E402, F401
from ocpeasy.compose import compose  # noqa E402, F401
from ocpeasy.destroy import destroy  # noqa E402, F401

if __name__ == "__main__":
    sys.exit(fire.Fire())



from os import path, getenv
from .utils import removeTrailSlash
from .constants import OCPEASY_CONFIG_NAME
from .notify import missingConfigurationFile, missingStage
import yaml

from .ocUtils import destroyApplication


def destroy(stageId: str):
    projectDevPath = getenv("PROJECT_DEV_PATH", None)
    pathProject = "." if not projectDevPath else removeTrailSlash(projectDevPath)
    ocpPeasyConfigPath = f"{pathProject}/{OCPEASY_CONFIG_NAME}"

    if path.isfile(ocpPeasyConfigPath):
        with open(ocpPeasyConfigPath) as ocpPeasyConfigFile:
            deployConfigDict = yaml.load(ocpPeasyConfigFile, Loader=yaml.FullLoader)
            globalValues = dict(deployConfigDict)
            stage = next(
                (x for x in globalValues.get("stages") if x.get("stageId") == stageId),
                None,
            )
            if stage:
                destroyApplication(stage.get("ocpProject"), stage.get("containerId"))
            else:
                return missingStage()
    else:
        return missingConfigurationFile()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ocpeasy']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.12,<4.0.0',
 'PyYAML>=5.3.1,<6.0.0',
 'cryptography==3.4.5',
 'fire>=0.3.1,<0.4.0',
 'openshift-client==1.0.12',
 'sh>=1.14.1,<2.0.0',
 'simple-term-menu>=0.10.4,<0.11.0']

setup_kwargs = {
    'name': 'ocpeasy',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 'David Barrat',
    'author_email': 'david@barrat.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
